Package: FuseSOM
Title: A Correlation Based Multiview Self Organizing Maps Clustering
        For IMC Datasets
Version: 1.15.0
Authors@R: 
    person("Elijah", "Willie", , "ewil3501@uni.sydney.edu.au", role = c("aut", "cre"))
Description: A correlation-based multiview self-organizing map for the
        characterization of cell types in highly multiplexed in situ
        imaging cytometry assays (`FuseSOM`) is a tool for unsupervised
        clustering. `FuseSOM` is robust and achieves high accuracy by
        combining a `Self Organizing Map` architecture and a
        `Multiview` integration of correlation based metrics. This
        allows FuseSOM to cluster highly multiplexed in situ imaging
        cytometry assays.
License: GPL-2
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.2
Imports: psych, FCPS, analogue, coop, pheatmap, ggplotify, fastcluster,
        fpc, ggplot2, stringr, ggpubr, proxy, cluster, diptest,
        methods, SummarizedExperiment, stats, S4Vectors
LazyData: false
BuildResaveData: false
Depends: R (>= 4.2.0)
Suggests: knitr, BiocStyle, rmarkdown, SingleCellExperiment
VignetteBuilder: knitr
BugReports: https://github.com/ecool50/FuseSOM/issues
LinkingTo: Rcpp
biocViews: SingleCell, CellBasedAssays, Clustering, Spatial
Config/pak/sysreqs: cmake make libicu-dev libuv1-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:59:22 UTC
RemoteUrl: https://github.com/bioc/FuseSOM
RemoteRef: HEAD
RemoteSha: e6e84914d8f60031513b6e205c7abb684b9d1aa3
NeedsCompilation: yes
Packaged: 2026-05-30 08:25:43 UTC; root
Author: Elijah Willie [aut, cre]
Maintainer: Elijah Willie <ewil3501@uni.sydney.edu.au>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-30 08:29:18 UTC; unix
