# Generated by roxygen2: do not edit by hand

export(clusterPrototypes)
export(computeGridSize)
export(estimateNumCluster)
export(generatePrototypes)
export(markerHeatmap)
export(normaliseData)
export(optiPlot)
export(runFuseSOM)
import(ggplot2)
import(ggpubr)
import(stringr)
importFrom(FCPS,HierarchicalClustering)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,colData)
importFrom(analogue,fuse)
importFrom(coop,tcosine)
importFrom(ggplotify,as.ggplot)
importFrom(methods,is)
importFrom(pheatmap,pheatmap)
importFrom(proxy,dist)
importFrom(psych,cor2dist)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,cov.wt)
importFrom(stats,kmeans)
importFrom(stats,prcomp)
importFrom(stats,princomp)
importFrom(stats,quantile)
importFrom(stats,sd)
useDynLib(FuseSOM)
