#' FuseSOM
#' 
#' FuseSOM provides a pipeline for the clustering of highly
#' multiplexed in situ imaging cytometry assays. This pipeline uses
#' the Self Organizing Map architecture coupled with Multiview hierarchical
#' clustering. We also provide functions for normalisation and estimation
#' of the number of clusters.
#'
#' The FuseSOM package provides three categories of important functions:
#' foo, bar and baz.
#'
#'
#' @docType package
#' @name FuseSOM
#' @useDynLib FuseSOM
NULL
#> NULL
