#' IMC Breast Cancer Data
#' Data from A spatial atlas of breast cancer progression using MIBI-TOF
#' and tissue transcriptomics
#' @usage data(risom_dat)
#'
#' @format An object of class `"data.frame"`.
#' @aliases
#' diabetesData
#'
#' @keywords datasets
#'
#' @references T. Risom, et al.
#' Transition to invasive breast cancer is associated with progressive changes
#' in the structure and composition of tumor stroma Cell, 185 (2022), pp. 299-310
#' ([ScienceDirect](https://www.sciencedirect.com/science/article/pii/S0092867421014860?via%3Dihub))
#'
#' @source Mendeley Data, <https://data.mendeley.com/datasets/d87vg86zd8/3>
#'
#'
"risom_dat"
